<?php

namespace Drupal\Tests\book_moderation_sync\Functional;

use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the Book Moderation Sync functionality.
 *
 * @group book_moderation_sync
 */
class BookModerationSyncTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'book',
    'content_moderation',
    'book_moderation_sync',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create the 'chapter' content type.
    $this->drupalCreateContentType(['type' => 'chapter', 'name' => 'Chapter']);

    // Configure the 'chapter' content type as a book.
    $config = $this->config('book.settings');
    $allowed_types = $config->get('allowed_types');
    $allowed_types[] = 'chapter';
    $config->set('allowed_types', $allowed_types)->save();

    // Enable moderation for book and chapter content types.
    $workflow = $this->createEditorialWorkflow();
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'book');
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'chapter');
    $workflow->save();

    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer nodes',
      'view any unpublished content',
      'use editorial transition create_new_draft',
      'use editorial transition publish',
    ]);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests that moderation states are synchronized between books and chapters.
   */
  public function testModerationStateSync() {
    // Create a book.
    $book = $this->drupalCreateNode([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);

    // Create a chapter and associate it with the book.
    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
      'book' => [
        'bid' => $book->id(),
        'pid' => $book->id(), // Explicitly define parent ID.
      ],
      'moderation_state' => 'draft',
    ]);

    // Validate parent-child relationship.
    $this->assertEquals($book->id(), $chapter->get('book')['bid'], 'The chapter is correctly associated with the book.');
    $this->assertEquals($book->id(), $chapter->get('book')['pid'], 'The chapter has the correct parent ID (pid).');

    // Publish the book.
    $book->set('moderation_state', 'published');
    $book->save();

    // Reload the chapter and validate state synchronization.
    $chapter = Node::load($chapter->id());
    $this->assertEquals(
      'published',
      $chapter->get('moderation_state')->value,
      'The chapter moderation state is synchronized with the book.'
    );
  }
}
